
#include <stdlib.h>
#include <stdio.h>
#include <sys/times.h>
#include <time.h>

#include "supermatrix.h"
#include "ie1d.h"
#include "graphics.h"
#include "krylov.h"
#include "hcoarsening.h"
#include "lapack.h"

int
main(int argc, char** argv){
  int n,k,nr,nmin,i;
  double *rhs, *u, zeit;
  struct tms buffer;
  clock_t t1, t2;
  psupermatrix s;
  char buf[80];

  n = 10000;
  k = 10;
  nr = 2;
  nmin = 20;

  (void) printf("#=================================#\n");
  (void) printf("#     Introductory Example:       #\n");
  (void) printf("#     One-dimensional integral    #\n");
  (void) printf("#     equation with Newton        #\n");
  (void) printf("#     kernel and piecewise        #\n");
  (void) printf("#     constant basis functions    #\n");
  (void) printf("#=================================#\n");
  (void) printf("# Number of basis functions n = ");
  (void) fgets(buf, 80, stdin);
  (void) sscanf(buf, "%d", &n);
  if(n<1 || n>50000){
    n = 64;
    (void) printf("# changed to n = %d\n",n);
  }
  (void) printf("# Order of the Taylor polynomial k = "); 
  (void) fgets(buf, 80, stdin);
  (void) sscanf(buf, "%d", &k);
  if(k<1 || k>1024){
    k = 4;
    (void) printf("# changed to k = %d\n",k);
  }
  (void) printf("# Right hand side number (1 or 2) = ");
  (void) fgets(buf, 80, stdin);
  (void) sscanf(buf, "%d", &nr);
  if(nr<1 || nr>2){
    nr = 1;
    (void) printf("# changed to number %d\n",nr);
  }
  (void) printf("# minimal blocksize = ");
  (void) fgets(buf, 80, stdin);
  (void) sscanf(buf, "%d", &nmin);
  if(nmin<1){
    nmin = 1;
    (void) printf("# changed to nmin = %d\n",nmin);
  }
  
  (void) printf("# Assembling the matrix\n");
  t1 = times(&buffer);
  s = fill_supermatrix(n,k,nmin);
  t2 = times(&buffer);
  zeit = (((double)t2-t1)/CLK_TCK);
  (void) printf("# Time for the assembly was %.2f seconds.\n",zeit);

  (void) printf("# Printing the matrix structure to 'matrix.ps'\n");
  outputsvd_supermatrix(s, "matrix.ps", 0);

  (void) printf("# Assembling the right hand side\n");
  rhs = fill_rhs(n,nr);
  (void) printf("# |rhs|_2 = %.5e\n",norm2_lapack(n,rhs));
  (void) printf("# Printing the right hand side to 'rhs.ps'\n");
  print_vector("rhs.ps",rhs,n);

  u = (double*) malloc(n*sizeof(double));
  for(i=0; i<n; i++) u[i] = 0.0;
  (void) printf("# Solving the discrete system by the cg-method\n");
  t1 = times(&buffer);
  solve_cg_supermatrix(s, rhs, u, 1e-8, 500, 0x0, 0, 1);
  t2 = times(&buffer);
  zeit = (((double)t2-t1)/CLK_TCK);
  (void) printf("# Time to solve (cg) was %.2f seconds.\n",zeit);

  (void) printf("# Printing the solution to 'solution.ps'\n");
  print_vector("solution.ps",u,n);

  (void) printf("# Freeing matrix, solution, right hand side.\n");
  del_supermatrix(s);
  free(u);
  free(rhs);
  (void) printf("# Program finished.\n");
  (void) printf("#===============================================#\n");
  return 0;
}
