
/* ------------------------------------------------------------
   This is the file "aca.h" of the HLib package.

   Copyright 1999 Steffen Boerm and Lars Grasedyck
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef ACA
#define ACA

typedef enum {
  HLIB_ACA_DEFAULT,		/* Try to bound relative Euclidean error */
  HLIB_ACA_ABSOLUTE,		/* Try to bound absolute Frobenius error */
  HLIB_ACA_RELATIVE,		/* Try to bound relative Frobenius error */
  HLIB_ACA_ABSOLUTE2,		/* Try to bound absolute Euclidean error */
  HLIB_ACA_RELATIVE2		/* Try to bound relative Euclidean error */
} ACAStrategy;

#include "basic.h"
#include "rkmatrix.h"

/* Compute an adaptive cross approximation of a matrix. The return value will
   be the rank k <= kmax, the first k colums of A and B will be filled by
   the rank-k-approximation of the matrix.
   A: Row matrix, containing at least rows*kmax double variables
   B: Column matrix, containing at least cols*kmax double variables
   rows: Number of rows
   cols: Number of columns
   row_off: Row offset, rows [row_off ... row_off+rows-1] are considered
   col_off: Column offset, columns [col_off ... col_off+cols-1] are considered
   entry: Callback function for computing a matrix entry
   data: Additional data for callback function "entry"
   strategy: Truncation strategy
   kmax: Maximal rank
   eps: Tolerance for truncation */
int
aca_fill_block(double *A, double *B, int rows, int cols,
	       int row_off, int col_off,
	       double (*entry)(int row, int col, void *data), void *data,
	       int kmax, double eps, ACAStrategy strategy);
int
newaca_fill_block(double *A, double *B, int rows, int cols,
	       int row_off, int col_off,
	       double (*entry)(int row, int col, void *data), void *data,
	       int kmax, double eps, ACAStrategy strategy);

/* Compute an adaptive cross approximation of a matrix and store it in
   an existing rkmatrix
   entry: Function for computing a matrix entry (e.g., nearfield quadrature)
   data: Additional data for "entry" (e.g., grid description, quadrature parameters)
   eps: Desired precision
   firstrow: First row in this submatrix (i.e., the row offset)
   firstcol: First column in this submatrix (i.e., the column offset)
   r: Target rkmatrix */
void
aca2_rkmatrix(double (*entry)(int row, int col, void *data),
	      void *data, double eps,
	      int firstrow, int firstcol,
	      prkmatrix r, ACAStrategy strategy);

/* Compute an adaptive cross approximation of a matrix and store it in
   an existing rkmatrix, reallocating if necessary.
   entry: Function for computing a matrix entry (e.g., nearfield quadrature)
   data: Additional data for "entry" (e.g., grid description, quadrature parameters)
   eps: Desired precision
   kmax: Maximal rank
   firstrow: First row in this submatrix (i.e., the row offset)
   firstcol: First column in this submatrix (i.e., the column offset)
   buf: Pointer to pointer to buffer (cf. provide_array in basic.h)
   bufsize: Pointer to buffer size (cf. provide_array in basic.h)
   r: Target rkmatrix */
void
bufaca2_rkmatrix(double (*entry)(int row, int col, void *data),
		 void *data, double eps, int kmax,
		 int firstrow, int firstcol,
		 double **buf, int *bufsize,
		 prkmatrix r, ACAStrategy strategy);

#endif
