
/* ------------------------------------------------------------
   This is the file "basic.h" of the HLib package.

   Copyright 1999 Steffen Boerm and Lars Grasedyck
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef Basic
#define Basic

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __GNUC__
static double dsqr(double) __attribute__((unused,const));
static double dmax(double, double) __attribute__((unused,const));
static double dmin(double, double) __attribute__((unused,const));
static int isqr(int) __attribute__((unused,const));
static int imax(int, int) __attribute__((unused,const));
static int imin(int, int) __attribute__((unused,const));
#endif

/* Truncation strategies for adaptive algorithms */

typedef enum {
  HLIB_FROBENIUS_ABSOLUTE,	/* Ensure |A-A'|_F <= eps globally */
  HLIB_FROBENIUS_RELATIVE,	/* Truncate if |A-A'|_F <= eps * |A|_F locally */
  HLIB_EUCLIDEAN_RELATIVE	/* Truncate if |A-A'|_2 <= eps * |A|_2 locally */
} TruncationStrategy;

/* Compute the square of a double */

static double
dsqr(double x)
{
  return x*x;
}

/* Compute the square of an integer */

static int
isqr(int x)
{
  return x*x;
}

/* Compute the maximum of two doubles */

static double
dmax(double x, double y)
{
  return (x < y ? y : x);
}

/* Compute the maximum of two integers */

static int
imax(int x, int y)
{
  return (x < y ? y : x);
}

/* Compute the minimum of two doubles */

static double
dmin(double x, double y)
{
  return (x > y ? y : x);
}

/* Compute the minimum of two integers */

static int
imin(int x, int y)
{
  return (x > y ? y : x);
}

/* Raise a double to a given integer power (hopefully faster than "pow") */

double
ipow(double x, int y);

/* Clear a vector */

void
clear_vector(int n,		/* Dimension */
	     double *x);	/* Target vector x */

/* Fill a vector with a value */

void
fill_vector(int n,		/* Dimension */
	    double *x,		/* Target vector x */
	    double v);		/* Fill value */

/* Fill a vector with random values */

void
random_vector(int n,
	      double *x);

/* Provide temporary double array */

void
provide_array(int n,		/* Desired size of the array */
	      double **x,	/* Pointer to array */
	      int *old_n);	/* Pointer to old size */

/* Remove "\n" from end of string */

void
remove_newline(char *buf);

/* Find out whether stdout is a terminal */

int
stdout_is_terminal();

/* Simple progressbar */

void
progressbar(int mode, int step, int allsteps, void *data);

/* Simple progressbar giving the percentage of completed work */

void
percent_progressbar(int mode, int step, int allsteps, void *data);

/* Not-so-simple progressbar giving the estimated time of arrival */

void
eta_progressbar(int mode, int step, int allsteps, void *data);

#endif
