
/* ------------------------------------------------------------
   This is the file "blockcluster.c" of the HLib package.

   Copyright 1999 Lars Grasedyck and Steffen Boerm
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#include "basic.h"
#include "blockcluster.h"

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* ************************************************************
    Blockcluster
   ************************************************************ */    

pblockcluster
new_blockcluster(pcluster row, int rowsons, pcluster col, int colsons, int adm) {
  pblockcluster bc;
  int i;

  bc = (pblockcluster) malloc(sizeof(blockcluster));
  assert(bc != NULL);

  bc->row = row;
  bc->col = col;
  bc->adm = adm;
  bc->rowsons = rowsons;
  bc->colsons = colsons;

  if(rowsons > 0 && colsons > 0) {
    bc->son = (pblockcluster *) 
      malloc((size_t) sizeof(pblockcluster) * rowsons * colsons);
    for(i=0; i<rowsons * colsons; i++)
      bc->son[i] = NULL;
  } else {
    bc->son = NULL;
  }
  
  return bc;
}

void
del_blockcluster(pblockcluster bc) {
  int i;

  assert(bc != NULL);

  if(bc->son != NULL) {
    for(i=0; i < bc->rowsons * bc->colsons; i++) {
      del_blockcluster(bc->son[i]);
      bc->son[i] = NULL;
    }

    free(bc->son);
    bc->son = NULL;
  }
  free(bc);
}

int
getnrnodes_blockcluster(pcblockcluster bc)
{
  int i;
  int nr;

  nr = 1;
  if(bc->son != NULL)
    for(i=0; i<bc->rowsons * bc->colsons; i++)
      nr += getnrnodes_blockcluster(bc->son[i]);

  return nr;
}

pblockcluster
build_blockcluster(pcluster row, pcluster col, double eta, int minmax,
		   int uniform, int nmin, int hom)
{
  pblockcluster bc;
  double dist = distance_cluster(row,col);
  double diam_row = diameter_cluster(row);
  double diam_col = diameter_cluster(col);
  int adm, etaadm, i,j;

  adm = (uniform == 1 ? 3 : 1);

  etaadm = (minmax == 0 ?
	    (diam_row < eta*dist || diam_col < eta*dist) :
	    (diam_row < eta*dist && diam_col < eta*dist));

  bc = NULL;

  if(etaadm == 1) {
    bc = new_blockcluster(row, 0, col, 0, adm);
  } else {
    if((minmax == 0 && (row->size <= nmin || col->size <= nmin)) ||
       (minmax == 1 && (row->size <= nmin && col->size <= nmin))) {
      bc = new_blockcluster(row, 0, col, 0, 0);
    } else {
      if(hom == 1 || (row->sons > 0 && col->sons > 0)) {
	bc = new_blockcluster(row, row->sons, col, col->sons, 0);
	for(i=0; i<row->sons; i++)
	  for(j=0; j<col->sons; j++)
	    bc->son[i+j*row->sons] = 
	      build_blockcluster(row->son[i], col->son[j],
				 eta, minmax, uniform, nmin, hom);
      } else {
	if(row->sons > 0 && col->sons == 0) {
	  bc = new_blockcluster(row, row->sons, col, 1, 0);
	  for(i=0; i<row->sons; i++)
	    bc->son[i] = 
	      build_blockcluster(row->son[i], col,
				 eta, minmax, uniform, nmin, hom);
	}
	if(row->sons == 0 && col->sons > 0){
	  bc = new_blockcluster(row, 1, col, col->sons, 0);
	    for(j=0; j<col->sons; j++)
	      bc->son[j] = 
		build_blockcluster(row, col->son[j],
				   eta, minmax, uniform, nmin, hom);
	}
	if(row->sons == 0 && col->sons == 0) {
	  bc = new_blockcluster(row, 0, col, 0, 0);
	}
      }
    } 
  }
  return bc;
}

pblockcluster
buildweak_blockcluster(pcluster row, pcluster col, double eta, int nmin){
  pblockcluster bc = 0x0;
  double dist = distance_cluster(row,col);
  double diam_row = diameter_cluster(row);
  double diam_col = diameter_cluster(col);
  int adm, etaadm, weakadm, i, j, dim;
  double mid;

  adm = 0;
  etaadm = 0;
  if(diam_row < eta*dist || diam_col < eta*dist){ 
    etaadm = 1;
    adm = 1;
  }
  weakadm = 0;
  
  dim = row->d;
  if(adm==0 && row->d==2 && (row->size<=10000 || col->size<=10000)){
    weakadm = 1;
    adm = 2;
    for(i=0; i<dim; i++){
      mid = 0.5*(row->bmin[i]+row->bmax[i]);
      if(mid > col->bmin[i] && mid < col->bmax[i]){
	weakadm=0;
	adm = 0;
      }
      mid = 0.5*(col->bmin[i]+col->bmax[i]);
      if(mid > row->bmin[i] && mid < row->bmax[i]){
	weakadm=0;
	adm = 0;
      }
    }
  }
  
  if(adm==0 && row->d>2 && (row->size<=10000 || col->size<=10000)){
    j = 0;
    for(i=0; i<dim; i++){
      if((0.5*(row->bmin[i]+row->bmax[i]) > col->bmin[i] && 
	  0.5*(row->bmin[i]+row->bmax[i]) < col->bmax[i] )
	 ||
	 (0.5*(col->bmin[i]+col->bmax[i]) > row->bmin[i] && 
	  0.5*(col->bmin[i]+col->bmax[i]) < row->bmax[i]) )
	j++;
    }
    if(j<1){
      weakadm = 1; 
      adm = 2;
    }
  }


  if(etaadm==1 || weakadm==1){
    bc = new_blockcluster(row, 0, col, 0, adm);
  }else{
    if(row->size <= nmin || col->size <= nmin){
      if((eta>10.1) && (row != col) && (row->size >3*nmin || col->size >3*nmin)){
	bc = new_blockcluster(row, 0, col, 0, 2);
      }else{
	bc = new_blockcluster(row, 0, col, 0, 0);
      }
    }else{
      if(row->sons>0 && col->sons>0){
	bc = new_blockcluster(row, row->sons, col, col->sons, 0);
	for(i=0; i<row->sons; i++)
	  for(j=0; j<col->sons; j++)
	    bc->son[i+j*row->sons] = 
	      buildweak_blockcluster(row->son[i], col->son[j],eta,nmin);
      }else{
	bc = new_blockcluster(row, 0, col, 0, 0);
      }
    } 
  }
  return bc;
}

pblockcluster
buildweakrec_blockcluster(pcluster row, pcluster col, double eta, int nmin){
  pblockcluster bc = 0x0;
  double dist = distance_cluster(row,col);
  double diam_row = diameter_cluster(row);
  double diam_col = diameter_cluster(col);
  int adm, etaadm, weakadm, i, j, dim;
  double mid;

  adm = 0;
  etaadm = 0;
  if(diam_row < eta*dist || diam_col < eta*dist){ 
    etaadm = 1;
    adm = 1;
  }
  weakadm = 0;
  
  dim = row->d;
  if(adm==0 && row->d==2 && (row->size<=10000 || col->size<=10000)){
    weakadm = 1;
    adm = 2;
    for(i=0; i<dim; i++){
      mid = 0.5*(row->bmin[i]+row->bmax[i]);
      if(mid > col->bmin[i] && mid < col->bmax[i]){
	weakadm=0;
	adm = 0;
      }
      mid = 0.5*(col->bmin[i]+col->bmax[i]);
      if(mid > row->bmin[i] && mid < row->bmax[i]){
	weakadm=0;
	adm = 0;
      }
    }
  }
  
  if(adm==0 && row->d>2 && (row->size<=10000 || col->size<=10000)){
    j = 0;
    for(i=0; i<dim; i++){
      if((0.5*(row->bmin[i]+row->bmax[i]) > col->bmin[i] && 
	  0.5*(row->bmin[i]+row->bmax[i]) < col->bmax[i] )
	 ||
	 (0.5*(col->bmin[i]+col->bmax[i]) > row->bmin[i] && 
	  0.5*(col->bmin[i]+col->bmax[i]) < row->bmax[i]) )
	j++;
    }
    if(j<1){
      weakadm = 1; 
      adm = 2;
    }
  }


  if(etaadm==1 || weakadm==1){
    bc = new_blockcluster(row, 0, col, 0, adm);
  }else{
    if(row->size <= nmin || col->size <= nmin){
      if((eta>10.1) && (row != col) && (row->size >3*nmin || col->size >3*nmin)){
	bc = new_blockcluster(row, 0, col, 0, 2);
      }else{
	bc = new_blockcluster(row, 0, col, 0, 0);
      }
    }else{
      if(row->sons>0 && col->sons>0){
	if(row->size > col->size){
	  bc = new_blockcluster(row, row->sons, col, 1, 0);
	  for(i=0; i<row->sons; i++)
	    bc->son[i] = buildweakrec_blockcluster(row->son[i], col,eta,nmin);
	}else{
	  bc = new_blockcluster(row, 1, col, col->sons, 0);
	  for(j=0; j<col->sons; j++)
	    bc->son[j] = buildweakrec_blockcluster(row, col->son[j],eta,nmin);
	}

      }else{
	bc = new_blockcluster(row, 0, col, 0, 0);
      }
    } 
  }
  return bc;
}

psupermatrix
buildsupermatrixfrom_blockcluster(pblockcluster bc, int k, double eps){
  psupermatrix s;
  int rows = bc->row->size;
  int cols = bc->col->size;
  int block_rows = bc->rowsons;
  int block_cols = bc->colsons;
  prkmatrix r = 0x0;
  pfullmatrix f = 0x0;
  int i,j;
  
  if(bc->son==0x0){
    if(bc->adm == 0 || rows*cols < k*(rows+cols)){
      f = new_fullmatrix(rows,cols);
    }else{
      r = new_rkmatrix(k,rows,cols);
      r->local_eps = eps;
    }
    s = new_supermatrix(1, 1, rows, cols, 0x0, r, f);
  }else{
    s = new_supermatrix(block_rows, block_cols, rows, cols, 0x0, r, f);
    for(i=0; i<block_rows; i++){
      for(j=0; j<block_cols; j++){
	s->s[i+j*block_rows] = 
	  buildsupermatrixfrom_blockcluster(bc->son[i+j*block_rows], k, eps);
      }
    }
  }
  return s;
}

void
setnr_cluster(pcluster c, int *size){
  int i;

  c->name = *size;
  *size = *size+1;
  for(i=0; i<c->sons; i++)
    setnr_cluster(c->son[i], size);
}

void
setnr_blockcluster(pblockcluster bc, int *size){
  int i;

  bc->name = *size;
  *size = *size+1;
  for(i=0; i<bc->rowsons*bc->colsons; i++)
    setnr_blockcluster(bc->son[i], size);
}

void
setarray_blockcluster(pblockcluster bc, pblockcluster *bcarray){
  int i;
  bcarray[bc->name] = bc;
  for(i=0; i<bc->rowsons*bc->colsons; i++)
    setarray_blockcluster(bc->son[i], bcarray);
}

void
setnrrowcol_blockcluster(pblockcluster bc, int *blocks_in_row, int *blocks_in_col){
  int i;
  blocks_in_row[bc->row->name]++;
  blocks_in_col[bc->col->name]++;
  for(i=0; i<bc->rowsons*bc->colsons; i++)
    setnrrowcol_blockcluster(bc->son[i], blocks_in_row, blocks_in_col);
}

void
setrowcol_blockcluster(pblockcluster bc, int *blocks_in_row, int *blocks_in_col,
		       int **row, int **col){
  int i;
  row[bc->row->name][blocks_in_row[bc->row->name]] = bc->col->name;
  blocks_in_row[bc->row->name]++;
  col[bc->col->name][blocks_in_col[bc->col->name]] = bc->row->name;
  blocks_in_col[bc->col->name]++;
  for(i=0; i<bc->rowsons*bc->colsons; i++)
    setrowcol_blockcluster(bc->son[i], blocks_in_row, blocks_in_col, row, col);  
}

pblockclustertree
new_blockclustertree(pblockcluster root){
  pblockclustertree bct;
  int i;
  int csp;
  
  bct = (pblockclustertree) malloc(sizeof(blockclustertree));
  assert(bct != 0x0);
  
  bct->root = root;
  bct->nr_blockcluster = 0;
  setnr_blockcluster(root,&bct->nr_blockcluster);
  bct->bcarray = 
    (pblockcluster*) malloc(bct->nr_blockcluster * sizeof(pblockcluster));
  setarray_blockcluster(root,bct->bcarray);
  
  bct->nr_rowcluster = 0;
  setnr_cluster(root->row,&bct->nr_rowcluster);
  bct->row = (int**) malloc(bct->nr_rowcluster * sizeof(int*));
  bct->blocks_in_row = (int*) malloc(bct->nr_rowcluster * sizeof(int));

  bct->nr_colcluster = 0;
  setnr_cluster(root->col,&bct->nr_colcluster);
  bct->col = (int**) malloc(bct->nr_colcluster * sizeof(int*));
  bct->blocks_in_col = (int*) malloc(bct->nr_rowcluster * sizeof(int));
  
  for(i=0; i<bct->nr_rowcluster; i++){
    bct->blocks_in_row[i] = 0;
  }
  for(i=0; i<bct->nr_colcluster; i++){
    bct->blocks_in_col[i] = 0;
  }

  setnrrowcol_blockcluster(root,bct->blocks_in_row,bct->blocks_in_col);
  csp = 0;
  for(i=0; i<bct->nr_rowcluster; i++){
    bct->row[i] = (int*) malloc(bct->blocks_in_row[i] * sizeof(int));
    if(bct->blocks_in_row[i]>csp) csp = bct->blocks_in_row[i];
    bct->blocks_in_row[i] = 0;
  }
  for(i=0; i<bct->nr_colcluster; i++){
    bct->col[i] = (int*) malloc(bct->blocks_in_col[i] * sizeof(int));
    if(bct->blocks_in_col[i]>csp) csp = bct->blocks_in_col[i];
    bct->blocks_in_col[i] = 0;
  }
  
  setrowcol_blockcluster(root,bct->blocks_in_row,bct->blocks_in_col,
			 bct->row,bct->col);

  return bct;
}

void
del_blockclustertree(pblockclustertree bct){
  int i;

  free(bct->bcarray);
  for(i=0; i<bct->nr_rowcluster; i++) free(bct->row[i]);
  for(i=0; i<bct->nr_colcluster; i++) free(bct->col[i]);
  free(bct->row);
  free(bct->col);
  free(bct->blocks_in_row);
  free(bct->blocks_in_col);
  free(bct);
}

