
/* ------------------------------------------------------------
   This is the file "blockcluster.h" of the HLib package.

   Copyright 1999 Lars Grasedyck and Steffen Boerm
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef Blockcluster
#define Blockcluster

typedef struct _blockcluster blockcluster;
typedef blockcluster *pblockcluster;
typedef const blockcluster *pcblockcluster;

typedef struct _blockclustertree blockclustertree;
typedef blockclustertree *pblockclustertree;
typedef const blockclustertree *pcblockclustertree;

#include "basic.h"
#include "cluster.h"
#include "supermatrix.h"

struct _blockcluster {
  int name;                     /* name = index in bcarray */
  int rowsons;			/* Number of sons in row */
  int colsons;			/* Number of sons in column */
  pblockcluster *son;		/* Array of sons */
  pcluster row;                 
  pcluster col;                 
  int adm;                      /* Admissibility flag, 
				   0: full/super inadmissible, 
				   1: rk admissible,
				   2: rk inadmissible,
				   3: uniform admissible */
  psupermatrix s;               /* Corresponding supermatrix */
};

/* Create a new blockcluster from a row and column cluster row,col
   with admissibility parameter adm (see above) */
pblockcluster
new_blockcluster(pcluster row, int rowsons, pcluster col, int colsons, int adm);

/* Free the memory used by a blockcluster object */
void
del_blockcluster(pblockcluster bc);

/* Returns the number of nodes in the block tree structure */
int
getnrnodes_blockcluster(pcblockcluster bc);

/* Create a block cluster tree from the roots row/col of two cluster trees.
   eta is the eta from the admissibility condition:
   minmax=0: min(diam,diam) <= eta dist
   minmax=1: max(diam,diam) <= eta dist
   uniform: uniformmatrix blocks instead of rkmatrix blocks
   nmin: minimal blocksize. For minmax=0 only blocks with 
    both clusters > nmin are subdivided, for minmax=1 this depends
    on hom.
   hom: construct a homogeneous block cluster tree if hom=1. 
*/
pblockcluster
build_blockcluster(pcluster row, pcluster col, double eta, int minmax,
		   int uniform, int nmin, int hom);

pblockcluster
buildweak_blockcluster(pcluster row, pcluster col, double eta, int nmin);

pblockcluster
buildweakrec_blockcluster(pcluster row, pcluster col, double eta, int nmin);

/* Create a supermatrix structure corresponding to the block cluster
   tree bc. Uniformmatrix blocks are replaced by rkmatrix blocks.
   k: Rank of the rkmatrix objects, eps: accuracy for adaptive rank arithmetic. 
*/
psupermatrix
buildsupermatrixfrom_blockcluster(pblockcluster bc, int k, double eps);

struct _blockclustertree {
  int nr_blockcluster;
  int nr_rowcluster;
  int nr_colcluster;
  pblockcluster root;		/* root of the tree */
  pblockcluster* bcarray;       /* array of blockclusters */
  int **row;                    /* rowwise array of blockcluster-names */
  int *blocks_in_row;           /* number of blocks in each row */
  int **col;                    /* columnwise array of blockcluster-names */
  int *blocks_in_col;           /* number of blocks in each column */
};

/* Create a new blockclustertree structure from a given tree (root) */
pblockclustertree
new_blockclustertree(pblockcluster root);

/* Free the memory used by a blockclustertree object */
void
del_blockclustertree(pblockclustertree bct);

#endif
