
/* ------------------------------------------------------------
   This is the file "curvebem.h" of the HLib package.

   Copyright 1999 Steffen Boerm and Lars Grasedyck
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef CurveBEM
#define CurveBEM

typedef struct _curvebemfactory curvebemfactory;
typedef curvebemfactory *pcurvebemfactory;
typedef const curvebemfactory *pccurvebemfactory;

#include "basic.h"
#include "cluster.h"
#include "clusterbasis.h"
#include "quadrature1d.h"
#include "supermatrix.h"

struct _curvebemfactory {
  int n;			/* Number of points on the closed polygon */
  const int *index;		/* Permutation */
  const double (*vertex)[2];	/* Array of points */

  double *xq;			/* Quadrature rule for regular integral */
  double *wq;
  int q;

  pquadrature1d q1;		/* Quadrature rules for singular integrals */

  double *xp;			/* Points for Chebyshev interpolation */
  int p;			/* Number of interpolation points */

  double (*l)[2];		/* Transformed 2D interpolation points */
  double (*l2)[2];
};

/* A curvebemfactory contains all the information required for the
   computation of an H- oder H^2-matrix approximation of the discretized
   single layer potential on a curve in 2D: An array of points describing
   the polygonal approximation of the curve, quadrature rules for singular
   and regular integrals as well as interpolation points used for the
   approximation of the kernel function. */

/* Create a curvebemfactory object
   n: Number of degrees of freedom
   index: Array of size n describing the permutation of the index set
   vertex: Array of points describing a closed polygon
   q: Order of the quadrature
   p: Order of the polynomial approximation */
pcurvebemfactory
new_curvebemfactory(int n,
		    const int *index, const double (*vertex)[2],
		    int q, int p);

/* Free the memory used by a curvebemfactory object */
void
del_curvebemfactory(pcurvebemfactory bfactory);

/* Compute the entries of an H-matrix approximation of the
   single layer potential
   M: H-matrix
   row: Cluster corresponding to the row of the matrix
   col: Cluster corresponding to the column of the matrix
   bfactory: Factory */
void
fill_curvebem_supermatrix(psupermatrix M, pccluster row, pccluster column,
			  pcurvebemfactory bfactory);

/* Create a nested cluster basis by integrating Lagrange polynomials
   t: Root cluster
   bfactory: Factory */
pclusterbasis
build_curvebem_clusterbasis(pccluster t, pcurvebemfactory bfactory);

/* Compute the entries of an H^2-matrix approximation of the
   single layer potential
   M: H^2-matrix
   row: Cluster corresponding to the row of the matrix
   col: Cluster corresponding to the column of the matrix
   bfactory: Factory */
void
fill_curvebem_supermatrix2(psupermatrix M, pccluster row, pccluster column,
			   pcurvebemfactory bfactory);

#endif
