
/* ------------------------------------------------------------
   This is the file "fem2d.h" of the HLib package.

   Copyright 1999 Steffen Boerm and Lars Grasedyck
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef FEM2D
#define FEM2D

typedef struct _grid2d grid2d;
typedef grid2d *pgrid2d;
typedef const grid2d *pcgrid2d;

#include "basic.h"
#include "cluster.h"
#include "sparsematrix.h"

#include <stdlib.h>

struct _grid2d {
  double (*p)[2];		/* Array of grid points */
  int (*t)[3];			/* Array of triangles */
  int *boundary;		/* Boundary flags (0 <=> interior point) */

  int *iblock;			/* Auxiliary memory */
  size_t isz;

  int vertices;			/* Number of grid points */
  int triangles;		/* Number of triangles */
};

/* Create an empty Grid2D object */
pgrid2d
new_grid2d(int vertices, int triangles);

/* Free the memory used by a Grid2D object */
void
del_grid2d(pgrid2d g2);

/* Build a geometric regular cluster tree for the grid defined by a Grid2D object,
   initialize perm and iperm arrays */
pclustertree
buildcluster_grid2d(pgrid2d g2, ClusterStrategy strategy, int leafsize);

/* Build the sparse matrix representing the discretized Laplace operator
   corresponding to a grid2d object */
psparsematrix
laplace_grid2d(pcgrid2d g2, pcclustertree ct);

/* Build a simple regular grid for the unit square */
pgrid2d
unitsquare_grid2d(int nx, int ny);

#endif
