
/* ------------------------------------------------------------
   This is the file "graphics.c" of the HLib package.

   Copyright 1999 Lars Grasedyck and Steffen Boerm
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#include "basic.h"
#include "graphics.h"

#include <stdio.h>

void
print_vector(char* name, double* v, int n){
  int i;
  double min,max;
  double scalex = n, scaley;
  FILE* datei;
  datei = fopen(name,"w");
  scalex = 500.0/scalex;

  min=1e+10;
  max=-1e+10;
  for(i=0; i<n;i++){
    if(v[i]<min) min = v[i];
    if(v[i]>max) max = v[i];
  }
  
  scaley = 1.0/(max-min);

  fprintf(datei,"%%!PS-Adobe-2.0-2.0 EPSF-2.0\n");
  fprintf(datei,"%%%%BoundingBox: 0 0 550 550\n");
  fprintf(datei,"25 dup translate\n");
  fprintf(datei,"%f dup scale\n",scalex);
  fprintf(datei,"%f setlinewidth\n",1.0/scalex);
  fprintf(datei,"/Helvetica findfont %f scalefont setfont\n",16.0/scalex);
  fprintf(datei,"/b{0 setgray} def\n");
  fprintf(datei,"/r{1.0 0.6 0.6  setrgbcolor} def\n");
  fprintf(datei,"/cs{closepath stroke} def\n");
  fprintf(datei,"/m{moveto} def\n");
  fprintf(datei,"/l{lineto} def\n");

  for(i=0; i<n; i++){
    fprintf(datei,"%d %f m\n",i,(v[i]-min)*n*scaley);
    fprintf(datei,"%d %f l\n",i+1,(v[i]-min)*n*scaley);
    fprintf(datei,"cs\n");
  }

  fprintf(datei,"r\n");
  fprintf(datei,"%d 0 m\n",-1);
  fprintf(datei,"%d 0 l\n",n+1);
  fprintf(datei,"cs\n");
  fprintf(datei,"b\n");
  fprintf(datei,"%d %f m\n",0, -0.04*n);
  fprintf(datei,"(0.0) show\n");
  fprintf(datei,"%d %f m\n",n, -0.04*n);
  fprintf(datei,"(1.0) show\n");
  
  fprintf(datei,"r\n");
  fprintf(datei,"%d %f m\n",0, -1.0);
  fprintf(datei,"%d %f l\n",0, n+1.0);
  fprintf(datei,"cs\n");
  fprintf(datei,"b\n");
  fprintf(datei,"%.2f %f m\n",-0.05*n, 0.0);
  fprintf(datei,"(%5.5f) show\n",min);
  fprintf(datei,"%.2f %f m\n",-0.05*n, 0.0+n);
  fprintf(datei,"(%5.5f) show\n",max);
  
  fprintf(datei,"showpage\n");
  fclose(datei);
}

void
printpart_supermatrix(FILE* datei, int nmax, int mmax, int nof, int mof, 
		      psupermatrix s){
  int i,j;
  int no,mo;
  int block_n = s->block_rows;
  int block_m = s->block_cols;
  int n,m;

  if(s->s!=0x0){
    no = nof;
    for(i=0; i<block_n; i++){
      mo = mof;
      for(j=0; j<block_m; j++){
	printpart_supermatrix(datei,nmax,mmax,no,mo,s->s[i+j*block_n]);
	mo += s->s[j*block_n]->cols; 
      }
      no += s->s[i]->rows;
    }
  }else{
    n = s->rows;
    m = s->cols;
    fprintf(datei,"%d %d m\n",mof,nmax-nof-n);
    fprintf(datei,"%d %d l\n",mof,nmax-nof-n+n);
    fprintf(datei,"%d %d l\n",mof+m,nmax-nof-n+n);
    fprintf(datei,"%d %d l\n",mof+m,nmax-nof-n);
    fprintf(datei,"%d %d l\n",mof,nmax-nof-n);
    if(s->r!=0x0){
      fprintf(datei,"fl\n");
    }else{
      fprintf(datei,"fr\n");
    }
    fprintf(datei,"cs\n");
    fprintf(datei,"b\n");
    fprintf(datei,"%d %d m\n",mof,nmax-nof-n);
    fprintf(datei,"%d %d l\n",mof,nmax-nof-n+n);
    fprintf(datei,"%d %d l\n",mof+m,nmax-nof-n+n);
    fprintf(datei,"%d %d l\n",mof+m,nmax-nof-n);
    fprintf(datei,"%d %d l\n",mof,nmax-nof-n);
    fprintf(datei,"cs\n");
  }
}

void 
print_supermatrix(char* name, psupermatrix s){
  double scale = s->rows;
  FILE* datei;
  datei = fopen(name,"w");
  scale = 500.0/scale;

  fprintf(datei,"%%!PS-Adobe-2.0-2.0 EPSF-2.0\n");
  fprintf(datei,"%%%%BoundingBox: 0 0 550 550\n");
  fprintf(datei,"%f dup translate\n",25.0);
  fprintf(datei,"%f dup scale\n",scale);
  fprintf(datei,"%f setlinewidth\n",0.2/scale);
  fprintf(datei,"/Helvetica findfont %f scalefont setfont\n",16.0/scale);
  fprintf(datei,"/b{0 setgray} def\n");
  fprintf(datei,"/fr{1.0 0.6 0.6  setrgbcolor fill} def\n");
  fprintf(datei,"/fl{0.6 1.0 0.6  setrgbcolor fill} def\n");
  fprintf(datei,"/cs{closepath stroke} def\n");
  fprintf(datei,"/m{moveto} def\n");
  fprintf(datei,"/l{lineto} def\n");

  printpart_supermatrix(datei,s->rows,s->cols,0,0,s);

  fprintf(datei,"showpage\n");
  fclose(datei);
}

