
/* ------------------------------------------------------------
   This is the file "lapack.h" of the HLib package.

   Copyright 1999 Steffen Boerm and Lars Grasedyck
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef Lapack
#define Lapack

#include "basic.h"

#include <math.h>

#ifdef __GNUC__
static void eval_lapack(int rows, int cols, const double *a, const double *x, double *y) __attribute__((unused));
static void addeval_lapack(int rows, int cols, const double *a, const double *x, double *y) __attribute__((unused));
static void evaltrans_lapack(int rows, int cols, const double *a, const double *x, double *y) __attribute__((unused));
static void addevaltrans_lapack(int rows, int cols, const double *a, const double *x, double *y) __attribute__((unused));
static void lowersolve_lapack(int n, const double *a, double *x) __attribute__((unused));
static void uppersolve_lapack(int n, const double *a, double *x) __attribute__((unused));
static void mul_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void multrans1_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void multrans2_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void multrans12_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void muladd_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void muladdtrans1_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void muladdtrans2_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void muladdtrans12_lapack(int rows, int cols, int mid, const double *a, const double *b, double *c) __attribute__((unused));
static void scale_lapack(int n, double *x, double v) __attribute__((unused));
static double norm2_lapack(int n, const double *x) __attribute__((unused));
static void copy_lapack(int n, const double *x, double *y) __attribute__((unused));
static void swap_lapack(int n, double *x, double *y) __attribute__((unused));
static void real_fft(int n, double *r, double *wsave) __attribute__((unused));
static void real_ifft(int n, double *r, double *wsave) __attribute__((unused));
static void complex_fft(int n, double *c, double *wsave) __attribute__((unused));
static void complex_ifft(int n, double *c, double *wsave) __attribute__((unused));
#endif

extern int eins_[1];
extern double deins_[1];
extern double dnull_[1];
extern double meins_[1];
extern char ttrans[1];
extern char ntrans[1];

/* ============================================================

     BLAS routines

   ============================================================ */

/* Compute y := y + alpha * x */

extern void
daxpy_(const int *n,		/* Dimension */
       const double *alpha,	/* Scaling factor for x */
       const double *x,		/* Source vector x */
       const int *incx,		/* Increment for x */
       double *y,		/* Target vector y */
       const int *incy);	/* Increment for y */

/* Copy x to y */

extern void
dcopy_(const int *n,		/* Dimension */
       const double *x,		/* Source vector x */
       const int *incx,		/* Increment for x */
       double *y,		/* Target vector y */
       const int *incy);	/* Increment for y */

static void
copy_lapack(int n, const double *x, double *y)
{
  dcopy_(&n, x, eins_, y, eins_);
}

/* Compute x := alpha * x */

extern void
dscal_(const int *n,		/* Dimension */
       const double *alpha,	/* Scaling factor */
       double *x,		/* Target vector x */
       const int *incx);	/* Increment for x */

static void
scale_lapack(int n, double *x, double v)
{
  dscal_(&n, &v, x, eins_);
}

/* Swap vector x and y */

extern void
dswap_(const int *n,		/* Dimension */
       double *x,		/* Target vector x */
       const int *incx,		/* Increment for x */
       double *y,		/* Target vector y */
       const int *incy);	/* Increment for y */

static void
swap_lapack(int n, double *x, double *y)
{
  dswap_(&n, x, eins_, y, eins_);
}

/* Compute the Euclidean inner product of vectors x and y */

extern double
ddot_(const int *n,		/* Dimension */
      const double *x,		/* Source vector x */
      const int *incx,		/* Increment for x */
      const double *y,		/* Source Vector y */
      const int *incy);		/* Increment for y */

/* Compute the norm of a vector x */

extern double
dnrm2_(const int *n,		/* Dimension */
       const double *x,		/* Vector x */
       const int *incx);	/* Increment for x */

static double
norm2_lapack(int n, const double *x)
{
  return dnrm2_(&n, x, eins_);
}

/* Compute the sum of the absolute values of the components of x */

extern double
dasum_(const int *n,		/* Dimension */
       const double *x,		/* Vector x */
       const int *incx);	/* Increment for x */

/* Compute the matrix-vector product y := beta * y + alpha * A * x */

extern void
dgemv_(const char *trans,	/* Transpose A? */
       const int *m,		/* Number of rows of A */
       const int *n,		/* Number of columns of A */
       const double *alpha,	/* Scaling factor for A*x */
       const double *a,		/* Matrix A */
       const int *lda,		/* Column length of A */
       const double *x,		/* Vector x */
       const int *incx,		/* Increment for x */
       const double *beta,	/* Scaling factor for y */
       double *y,		/* Result vector y */
       const int *incy);	/* Increment for y */

static void
eval_lapack(int rows, int cols, const double *a, const double *x, double *y)
{
  dgemv_("Not Transposed", &rows, &cols, deins_,
	 a, &rows, x, eins_, dnull_, y, eins_);
}

static void
addeval_lapack(int rows, int cols, const double *a, const double *x, double *y)
{
  dgemv_("Not Transposed", &rows, &cols, deins_,
	 a, &rows, x, eins_, deins_, y, eins_);
}

static void
evaltrans_lapack(int rows, int cols, const double *a, const double *x, double *y)
{
  dgemv_("Transposed", &rows, &cols, deins_,
	 a, &rows, x, eins_, dnull_, y, eins_);
}

static void
addevaltrans_lapack(int rows, int cols, const double *a, const double *x, double *y)
{
  dgemv_("Transposed", &rows, &cols, deins_,
	 a, &rows, x, eins_, deins_, y, eins_);
}

/* Solve a triangular system of linear equations */

extern void
dtrsv_(const char *uplo,	/* 'L': Lower triangular, 'U': Upper triangular */
       const char *trans,	/* Transposed A? */
       const char *diag,	/* 'U': Unit diagonal, 'N': Non-unit diagonal */
       const int *n,		/* Dimension */
       const double *a,		/* Matrix A */
       const int *lda,		/* Column length of A */
       double *x,		/* Input: Right-hand side, Output: Solution */
       const int *incx);	/* Increment for x */

extern void
dtrmv_(const char *uplo,	/* 'L': Lower triangular, 'U': Upper triangular */
       const char *trans,	/* Transposed A? */
       const char *diag,	/* 'U': Unit diagonal, 'N': Non-unit diagonal */
       const int *n,		/* Dimension */
       const double *a,		/* Matrix A */
       const int *lda,		/* Column length of A */
       double *x,		/* Input: Right-hand side, Output: Solution */
       const int *incx);	/* Increment for x */

static void
lowersolve_lapack(int n, const double *a, double *x)
{
  dtrsv_("Lower", "Not Transposed", "Non-unit diagonal", &n, a, &n, x, eins_);
}

static void
uppersolve_lapack(int n, const double *a, double *x)
{
  dtrsv_("Upper", "Not Transposed", "Non-unit diagonal", &n, a, &n, x, eins_);
}

/* Compute the matrix-matrix product C := beta * C + alpha * A * B */

extern void
dgemm_(const char *transa,	/* Transpose A? */
       const char *transb,	/* Transpose B? */
       const int *m,		/* Number of rows of the result C */
       const int *n,		/* Number of columns of the result C */
       const int *k,		/* Number of columns of A and rows of B */
       const double *alpha,	/* Scaling factor for A*B */
       const double *a,		/* Matrix A */
       const int *lda,		/* Column length of A */
       const double *b,		/* Matrix B */
       const int *ldb,		/* Column length of B */
       const double *beta,	/* Scaling factor for C */
       double *c,		/* Result matrix C */
       const int *ldc);		/* Spaltenlaenge von C */

static void
mul_lapack(int rows, int cols, int mid,
	   const double *a, const double *b, double *c)
{
  dgemm_("Not Transposed", "Not Transposed", &rows, &cols, &mid,
	 deins_, a, &rows, b, &mid, dnull_, c, &rows);
}

static void
multrans1_lapack(int rows, int cols, int mid,
		 const double *a, const double *b, double *c)
{
  dgemm_("Transposed", "Not Transposed", &rows, &cols, &mid,
	 deins_, a, &mid, b, &mid, dnull_, c, &rows);
}

static void
multrans2_lapack(int rows, int cols, int mid,
		 const double *a, const double *b, double *c)
{
  dgemm_("Not Transposed", "Transposed", &rows, &cols, &mid,
	 deins_, a, &rows, b, &cols, dnull_, c, &rows);
}

static void
multrans12_lapack(int rows, int cols, int mid,
		  const double *a, const double *b, double *c)
{
  dgemm_("Transposed", "Transposed", &rows, &cols, &mid,
	 deins_, a, &mid, b, &cols, dnull_, c, &rows);
}

static void
muladd_lapack(int rows, int cols, int mid,
	      const double *a, const double *b, double *c)
{
  dgemm_("Not Transposed", "Not Transposed", &rows, &cols, &mid,
	 deins_, a, &rows, b, &mid, deins_, c, &rows);
}

static void
muladdtrans1_lapack(int rows, int cols, int mid,
		    const double *a, const double *b, double *c)
{
  dgemm_("Transposed", "Not Transposed", &rows, &cols, &mid,
	 deins_, a, &mid, b, &mid, deins_, c, &rows);
}

static void
muladdtrans2_lapack(int rows, int cols, int mid,
		    const double *a, const double *b, double *c)
{
  dgemm_("Not Transposed", "Transposed", &rows, &cols, &mid,
	 deins_, a, &rows, b, &cols, deins_, c, &rows);
}

static void
muladdtrans12_lapack(int rows, int cols, int mid,
		     const double *a, const double *b, double *c)
{
  dgemm_("Transposed", "Transposed", &rows, &cols, &mid,
	 deins_, a, &mid, b, &cols, deins_, c, &rows);
}

/* Solve the system A*X=B of linear equations for a tridiagonal matrix A */

extern void
dgtsv_(const int *n,		/* Dimension */
       const int *nrhs,		/* Number of right hand sides */
       double *dl,		/* Subdiagonal elements */
       double *d,		/* Diagonal elements */
       double *du,		/* Superdiagonal elements */
       double *b,		/* Right hand sides B */
       const int *ldb,		/* Column length of B */
       int *info);		/* Result code */

/* ============================================================

     LAPACK routines

   ============================================================ */

/* Solve the system A*X=B of linear equations */

extern void
dgesv_(const int *n,		/* Dimension */
       const int *nrhs,		/* Number of right hand sides */
       double *a,		/* Matrix A */
       const int *lda,		/* Column length of A */
       int *ipiv,		/* Pivot indices, >= n elements */
       double *b,		/* Right hand sides B */
       const int *ldb,		/* Column length of B */
       int *info);		/* Result code */

/* Compute the Cholesky decomposition of a matrix A */

extern void
dpotrf_(const char *uplo,	/* 'U': Upper half given, 'L': Lower half */
	const int *n,		/* Dimension */
	double *a,		/* Input: Matrix A, Output: Cholesky factor */
	const int *lda,		/* Column length of A */
	int *info);		/* Result code */

/* Compute the LU decomposition A = P*L*U */

extern void
dgetrf_(const int *m,		/* Number of rows of A */
	const int *n,		/* Number of columns of A */
	double *a,		/* Input: Matrix to be factored, Output: Factors L and U, diagonal elements of L are not stored */
	const int *lda,		/* Column length of A */
	int *ipiv,		/* Pivot elements, >= min(n,m) elements */
	int *info);		/* Result code */

/* Use the LU decomposition to compute the inverse of A */

extern void
dgetri_(const int *n,		/* Dimension of the matrix */
	double *a,		/* Input: Factors L and U from dgetrf, Output: Inverse of A */
	const int *lda,		/* Column length of A */
	const int *ipiv,	/* Pivot elements form dgetrf */
	double *work,		/* Workspace, >= lwork elements */
	const int *lwork,	/* Size of workspace >= max(1,n) */
	int *info);		/* Result code */

/* Compute the QR factorization A = Q*R */

extern void
dgeqrf_(const int *m,		/* Number of rows of A */
	const int *n,		/* Number of columns of A */
	double *A,		/* Input: m by n matrix, Output: Entries on and above the diagonal are the factor R, entries below the diagonal define elementary reflectors */
	const int *lda,		/* Column length of A */
	double *tau,		/* Scalar factors of the elementary reflectors */
	double *work,		/* Workspace, >= lwork elements */
	const int *lwork,	/* Size of workspace, >= max(1,n) elements */
	int *info);		/* Result code */

/* Compute the factor Q of the QR factorization */

extern void
dorgqr_(const int *m,		/* Number of rows of Q */
	const int *n,		/* Number of columns of Q, m >= n >= 0 */
	const int *k,		/* Number of elementary reflectors defining Q, n >= k >= 0*/
	double *A,		/* Input: Elementary reflectors as returned by DGEQRF, Output: Matrix Q */
	const int *lda,		/* Column length of A */
	const double *tau,	/* Scalar factors of the elementary reflectors */
	double *work,		/* Workspace, >= lwork elements */
	const int *lwork,	/* Size of workspace, >= max(1,n) */
	int *info);		/* Result code */

/* Multiply a matrix C by the factor Q of the QR factorization */

extern void
dormqr_(const char *side,	/* Multiply Q from 'L'eft or 'R'ight */
	const char *trans,	/* 'N': Multiply by Q, 'T': Multiply by Q^T */
	const int *m,		/* Number of rows of C */
	const int *n,		/* Number of columns of C */
	const int *k,		/* Number of elementary reflectors, <= (side='L' ? m : n) */
	const double *a,	/* Elementary reflectors as returned by DGEQRF */
	const int *lda,		/* Column length of A */
	const double *tau,	/* Scalar factors of the elementary reflectors */
	double *c,		/* Input: Matrix C, Output: Q C or Q^T C or C Q or C Q^T */
	const int *ldc,		/* Column length of C */
	double *work,		/* Workspace, >= lwork elements */
	const int *lwork,	/* Size of workspace, >= (side='L' ? n : m) */
	int *info);

/* Solve a symmetric tridiagonal eigenvalue problem */

extern void
dstev_(const char *jobz,	/* 'N': Only eigenvalues, 'V': Eigenvectors, too */
       const int *n,		/* Dimension */
       double *d,		/* Input: Diagonal, Output: Eigenvalues */
       double *e,		/* Subdiagonal */
       double *z,		/* Eigenvectors, if jobz=V */
       int *ldz,		/* Column length of Z, >=n */
       double *work,		/* Workspace >=2*n-2, if jobz=V */
       int *info);		/* Result code */

/* Solve a symmetric eigenvalue problem */

extern void
dsyev_(const char *jobz,	/* 'N': Only eigenvalues, 'V': Eigenvectors, too */
       const char *uplo,	/* 'U': Upper half given, 'L': Lower half */
       const int *n,		/* Dimension */
       double *a,		/* Input: Matrix, Output: Eigenvectors if jobz=V */
       const int *lda,		/* Column length of A */
       double *w,		/* Eigenvalues sorted in increasing order */
       double *work,		/* Workspace */
       int *lwork,		/* Size of workspace, >=3*n-1 */
       int *info);		/* Result code */

/* Solve a general real eigenvalue problem */

extern void
dgeev_(const char *jobvl,	/* 'V': Compute left eigenvectors */
       const char *jobvr,	/* 'V': Compute right eigenvectors */
       const int *n,		/* Dimension */
       double *a,		/* Input: Matrix */
       const int *lda,		/* Column length of A */
       double *wr,		/* Real parts of the eigenvalues */
       double *wi,		/* Imaginary parts of the eigenvalues */
       double *vl,		/* Left eigenvectors */
       const int *ldvl,		/* Column length of VL */
       double *vr,		/* Right eigenvectors */
       const int *ldvr,		/* Column length of VR */
       double *work,		/* Workspace, size >= lwork */
       const int *lwork,	/* Size of workspace, >= 4*n */
       int *info);		/* Result code */

/* Compute the singular value decomposition */

extern void
dgesvd_(const char *jobu,	/* Is U to be computed? Where to store it? */
	const char *jobvt,	/* Is V to be computed? Where to store it? */
	const int *m,		/* Number of rows of A */
	const int *n,		/* Number of columns of A */
	double *a,		/* Input: Matrix A */
	const int *lda,		/* Column length of A */
	double *s,		/* Singular values of A, in decreasing order */
	double *u,		/* Output: Matrix U */
	const int *ldu,		/* Column length of U */
	double *vt,		/* Output: Matrix V */
	const int *ldvt,	/* Column length of V */
	double *work,		/* Workspace, >= lwork */
	const int *lwork,	/* Size of workspace, >= max(3*min(m,n)+max(m,n), 5*min(m,n)) */
	int *info);		/* Result code */

/* ============================================================

     FFTPACK routines

   ============================================================ */

/* Prepare workspace for real FFT and inverse FFT */

extern void
rffti_(const int *n,		/* Dimension */
       double *wsave);		/* Workspace, >= 2*n+15 doubles */

/* Perform real FFT */

extern void
rfftf_(const int *n,		/* Dimension */
       double *r,		/* Input: Real sequence, Output: Fourier coeffs */
       double *wsave);		/* Workspace as initialized by rffti */

static void
real_fft(int n, double *r, double *wsave)
{
  double norm = sqrt(1.0l / n);

  rfftf_(&n, r, wsave);
  dscal_(&n, &norm, r, eins_);
}

/* Perform real inverse FFT */

extern void
rfftb_(const int *n,		/* Dimension */
       double *r,		/* Input: Fourier coeffs, Output: Real sequence */
       double *wsave);		/* Workspace as initialized by rffti */

static void
real_ifft(int n, double *r, double *wsave)
{
  double norm = sqrt(1.0l / n);

  rfftb_(&n, r, wsave);
  dscal_(&n, &norm, r, eins_);
}

/* Prepare workspace for complex FFT and inverse FFT */

extern void
cffti_(const int *n,		/* Dimension */
       double *wsave);		/* Workspace, >= 4*n+15 doubles*/

/* Perform complex FFT */

extern void
cfftf_(const int *n,		/* Dimension */
       double *c,		/* Input: Complex sequence, Output: Fourier coeffs */
       double *wsave);		/* Workspace as initialized by cffti */

static void
complex_fft(int n, double *c, double *wsave)
{
  double norm = sqrt(1.0l / n);
  int n2 = 2 * n;

  cfftf_(&n, c, wsave);
  dscal_(&n2, &norm, c, eins_);
}

/* Perform complex inverse FFT */

extern void
cfftb_(const int *n,		/* Dimension */
       double *c,		/* Input: Fourier coeffs, Output: Complex sequence */
       double *wsave);		/* Workspace as initialized by cffti */

static void
complex_ifft(int n, double *c, double *wsave)
{
  double norm = sqrt(1.0l / n);
  int n2 = 2 * n;

  cfftb_(&n, c, wsave);
  dscal_(&n2, &norm, c, eins_);
}

#endif


