
/* ------------------------------------------------------------
   This is the file "net3d.h" of the HLib package.

   Copyright 1999 Lars Grasedyck and Steffen Boerm
   Max-Planck-Institut fuer Mathematik in den Naturwissenschaften
   Inselstrasse 22-26
   04103 Leipzig
   http://www.mis.mpg.de
   ------------------------------------------------------------ */

#ifndef NET3D
#define NET3D

typedef struct _net3d net3d;
typedef net3d *pnet3d;
typedef const net3d *pcnet3d;

typedef struct _net2d net2d;
typedef net2d *pnet2d;
typedef const net2d *pcnet2d;

#include "basic.h"
#include "cluster.h"
#include "sparsematrix.h"

#include <stdlib.h>

struct _net3d {
  double (*p)[3];		/* Array of grid points */

  int *perm;			/* Index array for grid points */
  int *iperm;			/* Inverse index array */

  int vertices;			/* Number of grid points */
};

struct _net2d {
  double (*p)[2];		/* Array of grid points */

  int *perm;			/* Index array for grid points */
  int *iperm;			/* Inverse index array */

  int vertices;			/* Number of grid points */
};

/********************** 3D **********************/

/* Create an empty net3d */
pnet3d
new_net3d(int vertices);

/* Create a net3d of a regular tensorgrid in 3d
   with verticesperaxis interior nodes per axis */
pnet3d
buildunitcube_net3d(int verticesperaxis);

psparsematrix
buildsparsematrixunitcube_net3d(pnet3d n3, double conv);

/* Free the memory used by a net3d object */
void
del_net3d(pnet3d n3);

/* Build a geometric regular cluster tree for a net3d */
pclustertree
buildcluster_net3d(pnet3d n3, ClusterStrategy strategy, int leafsize);

/********************** 2D **********************/

/* Create an empty net3d */
pnet2d
new_net2d(int vertices);

/* Create a net2d of a regular tensorgrid in 2d
   with verticesperaxis interior nodes per axis */
pnet2d
buildunitsquare_net2d(int verticesperaxis);

psparsematrix
buildsparsematrixunitsquare_net2d(pnet2d n2, double conv);

/* Free the memory used by a net2d object */
void
del_net2d(pnet2d n2);

/* Build a geometric regular cluster tree for a net2d */
pclustertree
buildcluster_net2d(pnet2d n2, ClusterStrategy strategy, int leafsize);

#endif
