
# ------------------------------------------------------------
# Copyright 2001 Steffen Boerm
# Max Planck Institute for Mathematics in the Sciences
# Inselstrasse 22-26
# 04103 Leipzig
# http://www.mis.mpg.de
# ------------------------------------------------------------

# AM_ARCHITECTURE
# Determine command-line parameters for given target architecture
AC_DEFUN(AM_ARCHITECTURE,
[
  AC_REQUIRE([AC_PROG_CC])
  AC_PROVIDE([AM_ARCHITECTURE])

  AC_ARG_ENABLE(arch, [  --enable-arch=ARCH      Compile for given architecture (optional)], [
    if ( echo $enableval | grep v8 > /dev/null ) ; then
      gcc_cflags="-mcpu=v8"
      gcc_ldflags="-Xcompiler -mcpu=v8"
    else
      if ( echo $enableval | grep v9 > /dev/null ) ; then
        gcc_cflags="-mcpu=v9 -m64"
        gcc_ldflags="-Xcompiler -mcpu=v9 -Xcompiler -m64"
      else
        gcc_flags="-mcpu=$enableval"
        gcc_ldflags="-Xcompiler -mcpu=$enableval"
      fi
    fi

    saved_CFLAGS="$CFLAGS"
    saved_LDFLAGS="$LDFLAGS"

    CFLAGS="$saved_CFLAGS -xarch=$enableval"
    LDFLAGS="$saved_LDFLAGS -xarch=$enableval"
    AC_MSG_CHECKING([whether -xarch=$enableval works])
    AC_TRY_LINK(, [/* nothing */], [
      AC_MSG_RESULT(yes)

      ARCH="$enableval"
    ], [
      AC_MSG_RESULT(no)

      CFLAGS="$saved_CFLAGS $gcc_cflags"
      LDFLAGS="$saved_LDFLAGS $gcc_ldflags"
      AC_MSG_CHECKING([whether $gcc_cflags works])
      AC_TRY_LINK(, [/* nothing */], [
	AC_MSG_RESULT(yes)

        ARCH="$enableval"
      ], [
        AC_MSG_RESULT([no, ignoring switch])

	ARCH=""
        CFLAGS="$saved_CFLAGS"
        LDFLAGS="$saved_LDFLAGS"
      ])
    ])

    AC_DEFINE_UNQUOTED(ARCHITECTURE, $ARCH, [Target architecture])

    AC_SUBST(ARCH)
  ], )
])

# AM_OPTIMIZATION
# Determine command-line parameters for code optimization
AC_DEFUN(AM_OPTIMIZATION,
[
  AC_REQUIRE([AC_PROG_CC])
  AC_PROVIDE([AM_OPTIMIZATION])

  AC_ARG_ENABLE(optimize, [  --enable-optimize       Enable optimization (optional)], [
    if test x$enableval = xyes ; then
      opti_wanted="full"
    else
      opti_wanted="no"
    fi
  ], [
    opti_wanted="yes"
  ])

  if test x$opti_wanted != xno ; then
    saved_CFLAGS="`echo ${CFLAGS} | sed "s/-g//g" | sed "s/-O2//g"`"

    if test x$opti_wanted = xfull ; then
      CFLAGS="-DNDEBUG $saved_CFLAGS"
      AC_MSG_CHECKING([whether -DNDEBUG works])
      AC_TRY_LINK(, [/* nothing */], [
        AC_MSG_RESULT(yes)

        saved_CFLAGS="${CFLAGS}"
      ], [
        AC_MSG_RESULT(no)
      ])
    fi

    CFLAGS="-fast $saved_CFLAGS"
    AC_MSG_CHECKING([whether -fast works])
    AC_TRY_LINK(, [/* nothing */], [
      AC_MSG_RESULT(yes)
    ], [
      AC_MSG_RESULT(no)

      CFLAGS="$saved_CFLAGS -xO5"
      AC_MSG_CHECKING([whether -xO5 works])
      AC_TRY_LINK(, [/* nothing */], [
        AC_MSG_RESULT(yes)
      ], [
        AC_MSG_RESULT(no)

        CFLAGS="$saved_CFLAGS -O3 -funroll-loops"
        AC_MSG_CHECKING([whether -O3 -funroll-loops works])
        AC_TRY_LINK(, [/* nothing */], [
          AC_MSG_RESULT(yes)
        ], [
          AC_MSG_RESULT(no)

          CFLAGS="$saved_CFLAGS -O"
          AC_MSG_CHECKING([whether -O works])
          AC_TRY_LINK(, [/* nothing */], [
            AC_MSG_RESULT(yes)
          ], [
            AC_MSG_RESULT([no, ignoring switch])

            CFLAGS="$saved_CFLAGS"
          ])
        ])
      ])
    ])
  fi      

  if test x$opti_wanted = xyes ; then
    AC_DEFINE_UNQUOTED(OPTIMIZATION, 1, [Optimization])
  else
    if test x$opti_wanted = xfull ; then
      AC_DEFINE_UNQUOTED(OPTIMIZATION, 2, [Optimization])
    fi
  fi
])

# AM_DEBUGGING
# Determine command-line parameters for debugging
AC_DEFUN(AM_DEBUGGING,
[
  AC_REQUIRE([AC_PROG_CC])
  AC_PROVIDE([AM_DEBUGGING])

  AC_ARG_ENABLE(debug, [  --enable-debug          Enable debugging (optional)], [
    if test x$enableval = xyes ; then
      saved_CFLAGS="`echo ${CFLAGS} | sed "s/-O2//g"`"

      CFLAGS="$saved_CFLAGS -g"
      AC_MSG_CHECKING([whether -g works])
      AC_TRY_LINK(, [/* nothing */], [
        AC_MSG_RESULT(yes)
        debug=yes
      ], [
        AC_MSG_RESULT([no, ignoring switch])

        CFLAGS="$saved_CFLAGS"
      ])

    fi
  ])

  saved_CFLAGS="$CFLAGS"

  CFLAGS="$saved_CFLAGS -Wall -pedantic"
  AC_MSG_CHECKING([whether -Wall -pedantic works])
  AC_TRY_LINK(, [/* nothing */], [
    AC_MSG_RESULT(yes)
    
    CFLAGS="$saved_CFLAGS -Wall -Wno-unknown-pragmas -pedantic"
    AC_MSG_CHECKING([whether -Wall -Wno-unknown-pragmas -pedantic works])
    AC_TRY_LINK(, [/* nothing */], [
      AC_MSG_RESULT(yes)
    ], [
      AC_MSG_RESULT(no)

      CFLAGS="$saved_CFLAGS -Wall -pedantic"
    ])
  ], [
    AC_MSG_RESULT(no)

    CFLAGS="$saved_CFLAGS"
  ])

  #saved_CFLAGS="$CFLAGS"

  #CFLAGS="$saved_CFLAGS -Werror"
  #AC_MSG_CHECKING([whether -Werror works])
  #AC_TRY_LINK(, [/* nothing */], [
  #  AC_MSG_RESULT(yes)
  #], [
  #  AC_MSG_RESULT(no)

  #  CFLAGS="$saved_CFLAGS -Werr"
  #  AC_MSG_CHECKING([whether -Werr works])
  #  AC_TRY_LINK(, [/* nothing */], [
  #    AC_MSG_RESULT(yes)
  #  ], [
  #    AC_MSG_RESULT(no)

  #    CFLAGS="$saved_CFLAGS"
  #  ])
  #])

  if test x$debug = xyes ; then
    AC_DEFINE_UNQUOTED(DEBUGGING, 1, [Debugging])
  fi
])

# AM_LINKRPATH
# Determine command-line parameters for hardcoding runpath in libraries
AC_DEFUN(AM_LINKRPATH,
[
  AC_PROVIDE([AM_LINKRPATH])

  ld_libpath="-L"
  ld_runpath="-R"

  if test -r ${HOME}/.sblibs ; then
    eval `grep ld_libpath= ${HOME}/.sblibs`
    eval `grep ld_runpath= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(ld-libpath, [  --with-ld-libpath=ARG   Linker switch for library search path (optional)], [
    ld_libpath="$withval"
  ], :)

  AC_ARG_WITH(ld-runpath, [  --with-ld-runpath=ARG   Linker switch for runtime library path (optional)], [
    ld_runpath="$withval"
  ], :)

  saved_LDFLAGS="$LDFLAGS"

  AC_MSG_CHECKING([search path for linker])
  LDFLAGS="-Wl,${ld_libpath},/ $saved_LDFLAGS"
  AC_TRY_LINK(, [ /* nothing */ ], [
    AC_MSG_RESULT(yes)

    LDFLAGS="$saved_LDFLAGS"
    have_libpath=yes
  ], [
    AC_MSG_RESULT(no)    

    LDFLAGS="$saved_LDFLAGS"
    have_libpath=no
    ifelse([$2], , :, [$2])
  ])

  AC_MSG_CHECKING([search path for runtime linker])
  LDFLAGS="-Wl,${ld_runpath},/ $saved_LDFLAGS"
  AC_TRY_LINK(, [ /* nothing */ ], [
    AC_MSG_RESULT(yes)

    LDFLAGS="$saved_LDFLAGS"
    have_runpath=yes
  ], [
    AC_MSG_RESULT(no)

    LDFLAGS="$saved_LDFLAGS"
    have_runpath=no
  ])

  if test $have_runpath = no ; then
    AC_MSG_CHECKING([whether -rpath works])
    LDFLAGS="-Wl,-rpath,/ $saved_LDFLAGS"
    AC_TRY_LINK(, [ /* nothing */ ], [
      AC_MSG_RESULT(yes)

      ld_runpath="-rpath"
      LDFLAGS="$saved_LDFLAGS"
      have_runpath=yes
    ], [
      AC_MSG_RESULT(no)
    ])
  fi

  if test $have_runpath = no ; then
    AC_MSG_CHECKING([ whether --rpath works])
    LDFLAGS="-Wl,--rpath,/ $saved_LDFLAGS"
    AC_TRY_LINK(, [ /* nothing */ ], [
      AC_MSG_RESULT(yes)

      ld_runpath="--rpath"
      LDFLAGS="$saved_LDFLAGS"
      have_runpath=yes
    ], [
      AC_MSG_RESULT(no)

      LDFLAGS="$saved_LDFLAGS"
      ifelse([$2], , :, [$2])
    ])
  fi

  AC_SUBST(ld_libpath)
  AC_SUBST(ld_runpath)

  ifelse([$1], , :, [$1])
])

# AM_PATH_BLAS([ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
# Determine command-line parameters for BLAS library
AC_DEFUN(AM_PATH_BLAS,
[
  AC_REQUIRE([AM_LINKRPATH])
  AC_PROVIDE([AM_PATH_BLAS])

  blas_libs=""
  blas_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep blas_libs= ${HOME}/.sblibs`
    eval `grep blas_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(blas-prefix, [  --with-blas-prefix=PFX  Prefix where BLAS is installed (optional)], [
    blas_ldflags="-Wl,${ld_libpath}`echo $withval | sed 's/:/ -Wl,${ld_libpath}/g'` -Wl,${ld_runpath}`echo $withval | sed 's/:/ -Wl,${ld_runpath}/g'`"
  ], :)

  AC_ARG_WITH(blas-libs,     [  --with-blas-libs=LIBS   BLAS libraries (optional)], [
    blas_libs="$withval"
  ], :)

  AC_ARG_WITH(blas-ldflags,  [  --with-blas-ldflags=ARG BLAS linker parameters (optional)], [
    blas_ldflags="$withval"
  ], :)

  saved_LIBS="$LIBS"
  saved_LDFLAGS="$LDFLAGS"

  #
  # User-defined
  #

  AC_MSG_CHECKING([for BLAS])
  LIBS="$blas_libs $saved_LIBS"
  LDFLAGS="$blas_ldflags $saved_LDFLAGS"
  AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
    AC_MSG_RESULT(yes)
    have_BLAS=yes

    BLAS_LIBS="$blas_libs"
  ], [
    AC_MSG_RESULT(no)
    have_BLAS=no
  ])

  #
  # Sun Compiler
  #

  if test $have_BLAS = no ; then
    LIBS="-xlic_lib=sunperf $saved_LIBS"
    LDFLAGS="$saved_LDFLAGS"
    AC_MSG_CHECKING([for BLAS in SUN Performance Library (using -xlic_lib)])
    AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
      AC_MSG_RESULT(yes)
      have_BLAS=yes

      BLAS_LIBS="-xlic_lib=sunperf"
      LIBS="$saved_LIBS"
    ], [
      AC_MSG_RESULT(no)
      have_BLAS=no
    ])
  fi

  #
  # Sun Performance Library
  #

  if test $have_BLAS = no ; then
    if test x$ARCH != x ; then
      if ( echo $ARCH | grep v9 > /dev/null ) ; then
        LDFLAGS="-Wl,${ld_libpath}/opt/SUNWspro/lib/$ARCH,${ld_libpath}/opt/SUNWspro/lib/v9,${ld_runpath}/opt/SUNWspro/lib/$ARCH,${ld_runpath}/opt/SUNWspro/lib/v9 $saved_LDFLAGS"
        if test -r /opt/SUNWspro/prod/lib/v9/stack_grow.o ; then
          stack_grow="/opt/SUNWspro/prod/lib/v9/stack_grow.o"
        else
          stack_grow=""
        fi
      else
        if ( echo $ARCH | grep v8 > /dev/null ) ; then
          LDFLAGS="-Wl,${ld_libpath}/opt/SUNWspro/lib/$ARCH,${ld_libpath}/opt/SUNWspro/lib,${ld_runpath}/opt/SUNWspro/lib/$ARCH,${ld_runpath}/opt/SUNWspro/lib $saved_LDFLAGS"
        else
          LDFLAGS="-Wl,${ld_libpath}/opt/SUNWspro/lib/$ARCH,${ld_runpath}/opt/SUNWspro/lib/$ARCH $saved_LDFLAGS"
        fi
        if test -r /opt/SUNWspro/prod/lib/stack_grow.o ; then
          stack_grow="/opt/SUNWspro/prod/lib/stack_grow.o"
        else
          stack_grow=""
        fi
      fi
    else
      LDFLAGS="-Wl,${ld_libpath}/opt/SUNWspro/lib,${ld_runpath}/opt/SUNWspro/lib"
      stack_grow=""
    fi
    LIBS="-lsunperf ${stack_grow} -lfsu -lF77 -lfai -lsunmath -lm $saved_LIBS"
 
    AC_MSG_CHECKING([for BLAS in SUN Performance Library])
    AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
      AC_MSG_RESULT(yes)
      have_BLAS=yes
      BLAS_LIBS="-lsunperf ${stack_grow} -lfsu -lF77 -lfai -lsunmath -lm"
      LIBS="$saved_LIBS"
    ], [
      AC_MSG_RESULT(no)
      have_BLAS=no
    ])
  fi

  #
  # Intel Math Kernel Library
  #

  if test $have_BLAS = no ; then
    mkldir="`find /opt/intel* -name "libmkl.so" 2> /dev/null | grep 32 | sed -e "s/\/libmkl.so//"`"
    AC_MSG_CHECKING([for BLAS in Intel Math Kernel Library])
    if test -n "$mkldir" ; then
      LIBS="-llapack -lg2c -lmkl -lguide -lpthread -lm $saved_LIBS"
      LDFLAGS="-Wl,-L,${mkldir},-R,${mkldir} $saved_LDFLAGS"
      AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
        AC_MSG_RESULT(yes)
        have_BLAS=yes

        BLAS_LIBS="-llapack -lg2c -lmkl -lguide -lpthread -lm"
        LIBS="$saved_LIBS"
      ], [
        AC_MSG_RESULT(no)
        have_BLAS=no
      ])
    else
      AC_MSG_RESULT(no)
    fi
  fi

  #
  # SGI Complib
  #

  if test $have_BLAS = no ; then
    LIBS="-lcomplib.sgimath -lm $saved_LIBS"
    LDFLAGS="$saved_LDFLAGS"
    AC_MSG_CHECKING([for BLAS in SGI Complib])
    AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
      AC_MSG_RESULT(yes)
      have_BLAS=yes

      BLAS_LIBS="-lcomplib.sgimath -lm"
      LIBS="$saved_LIBS"
    ], [
      AC_MSG_RESULT(no)
      have_BLAS=no
    ])
  fi

  #
  # Compaq Extended Math Library
  #

  if test $have_BLAS = no ; then
    LIBS="-lcxml -lm $saved_LIBS"
    LDFLAGS="$saved_LDFLAGS"
    AC_MSG_CHECKING([for BLAS in Compaq Extended Math Library])
    AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
      AC_MSG_RESULT(yes)
      have_BLAS=yes

      BLAS_LIBS="-lcxml -lm"
      LIBS="$saved_LIBS"
    ], [
      AC_MSG_RESULT(no)
      have_BLAS=no
    ])
  fi

  #
  # g77 BLAS
  #

  if test $have_BLAS = no ; then
    LIBS="-lblas -lg2c -lm $saved_LIBS"
    LDFLAGS="$saved_LDFLAGS"
    AC_MSG_CHECKING([for g77 BLAS])
    AC_TRY_LINK(, [ void dgemv_(); dgemv_(); ], [
      AC_MSG_RESULT(yes)
      have_BLAS=yes

      BLAS_LIBS="-lblas -lg2c -lm"
      LIBS="$saved_LIBS"
    ], [
      AC_MSG_RESULT(no)
      have_BLAS=no
    ])
  fi

  #
  # Clean-up
  #

  if test $have_BLAS = no ; then
    BLAS_LIBS=""
    LDFLAGS="$saved_LDFLAGS"
  fi

  LIBS="$saved_LIBS"

  if test x$have_BLAS != xyes ; then
    AC_SUBST(BLAS_LIBS)

    ifelse([$2], , :, [$2])
  else
    AC_DEFINE(HAVE_BLAS, 1, [Defined if BLAS was found])

    AC_SUBST(BLAS_LIBS)

    ifelse([$1], , :, [$1])
  fi
])

# AM_PATH_LAPACK([ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
# Determine command-line parameters for LAPACK library
AC_DEFUN(AM_PATH_LAPACK,
[
  AC_REQUIRE([AM_LINKRPATH])
  AC_REQUIRE([AM_PATH_BLAS])
  AC_PROVIDE([AM_PATH_LAPACK])

  lapack_libs="-llapack"
  lapack_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep lapack_libs= ${HOME}/.sblibs`
    eval `grep lapack_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(lapack-prefix, [  --with-lapack-prefix=PFX Prefix where LAPACK is installed (optional)], [
    lapack_ldflags="-Wl,${ld_libpath}$withval,${ld_runpath}$withval"
  ], :)

  AC_ARG_WITH(lapack-libs, [  --with-lapack-libs=ARG  LAPACK libraries (optional)], [
    lapack_libs="$withval"
  ], :)

  AC_ARG_WITH(lapack-ldflags, [  --with-lapack-ldflags=ARG LAPACK linker parameters (optional)], [
    lapack_ldflags="$withval"
  ], :)

  saved_LIBS="$LIBS"
  saved_LDFLAGS="$LDFLAGS"

  LDFLAGS="$lapack_ldflags $saved_LDFLAGS"

  # Test for dsyev
  AC_MSG_CHECKING([for implicit LAPACK])
  LIBS="$BLAS_LIBS $saved_LIBS"
  
  AC_TRY_LINK(, [ void dsyev_(); dsyev_(); ], [
    AC_MSG_RESULT(yes)
    have_LAPACK=yes
    LAPACK_LIBS=""
  ], [
    AC_MSG_RESULT(no)
    AC_MSG_CHECKING([for standard LAPACK])
    LIBS="$lapack_libs $BLAS_LIBS $saved_LIBS"
    AC_TRY_LINK(, [ void dsyev_(); dsyev_(); ], [
      AC_MSG_RESULT(yes)
      have_LAPACK=yes

      LAPACK_LIBS="$lapack_libs"
    ], [
      AC_MSG_RESULT(no)

      AC_MSG_CHECKING([LAPACK in SUN Performance Library])
      LIBS="-lsunperf $BLAS_LIBS $saved_LIBS"
      AC_TRY_LINK(, [ void dsyev_(); dsyev_(); ], [
        AC_MSG_RESULT(yes)
        have_LAPACK=yes

        LAPACK_LIBS="-lsunperf"
      ], [
        AC_MSG_RESULT(no)
        have_LAPACK=no

        LDFLAGS="$saved_LDFLAGS"
      ])
    ])
  ])

  LIBS="$saved_LIBS"

  if test x$have_LAPACK != xyes ; then
    AC_SUBST(LAPACK_LIBS)

    ifelse([$2], , :, [$2])
  else
    AC_DEFINE(HAVE_LAPACK, 1, [Defined if LAPACK was found])

    AC_SUBST(LAPACK_LIBS)

    ifelse([$1], , :, [$1])
  fi
])

# AM_PATH_FFTPACK([ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
# Determine command-line parameters for FFTPACK library
AC_DEFUN(AM_PATH_FFTPACK,
[
  AC_REQUIRE([AM_LINKRPATH])
  AC_REQUIRE([AM_PATH_BLAS])
  AC_PROVIDE([AM_PATH_FFTPACK])

  fftpack_libs="-lfftpack"
  fftpack_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep fftpack_libs= ${HOME}/.sblibs`
    eval `grep fftpack_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(fftpack-prefix, [  --with-fftpack-prefix=PFX Prefix where FFTPACK is installed (optional)], [
    fftpack_ldflags="-Wl,${ld_libpath}$withval,${ld_runpath}$withval"
  ], :)

  AC_ARG_WITH(fftpack-libs, [  --with-fftpack-libs=ARG FFTPACK libraries (optional)], [
    fftpack_libs="$withval"
  ], :)

  AC_ARG_WITH(fftpack-ldflags, [  --with-fftpack-ldflags=ARG FFTPACK linker parameters (optional)], [
    fftpack_ldflags="$withval"
  ], :)

  saved_LIBS="$LIBS"
  saved_LDFLAGS="$LDFLAGS"

  LDFLAGS="$fftpack_ldflags $saved_LDFLAGS"

  # Test for dsyev
  AC_MSG_CHECKING([for implicit FFTPACK])
  LIBS="$BLAS_LIBS $saved_LIBS"
  
  AC_TRY_LINK(, [ void rfftf_(); rfftf_(); ], [
    AC_MSG_RESULT(yes)
    have_FFTPACK=yes
    FFTPACK_LIBS=""
  ], [
    AC_MSG_RESULT(no)
    AC_MSG_CHECKING([for standard FFTPACK])
    LIBS="$fftpack_libs $BLAS_LIBS $saved_LIBS"
    AC_TRY_LINK(, [ void rfftf_(); rfftf_(); ], [
      AC_MSG_RESULT(yes)
      have_FFTPACK=yes

      FFTPACK_LIBS="$fftpack_libs"
    ], [
      AC_MSG_RESULT(no)

      AC_MSG_CHECKING([for FFTPACK in SUN Performance Library])
      LIBS="-lsunperf $BLAS_LIBS $saved_LIBS"
      AC_TRY_LINK(, [ void dsyev_(); dsyev_(); ], [
        AC_MSG_RESULT(yes)
        have_FFTPACK=yes

        FFTPACK_LIBS="-lsunperf"
      ], [
        AC_MSG_RESULT(no)
        have_FFTPACK=no

        LDFLAGS="$saved_LDFLAGS"
      ])
    ])
  ])

  LIBS="$saved_LIBS"

  if test x$have_FFTPACK != xyes ; then
    AC_SUBST(FFTPACK_LIBS)

    ifelse([$2], , :, [$2])
  else
    AC_DEFINE(HAVE_FFTPACK, 1, [Defined if FFTPACK was found])

    AC_SUBST(FFTPACK_LIBS)

    ifelse([$1], , :, [$1])
  fi
])

# AM_PATH_GLUT([ACTION-IF-FOUND] [, ACTION-IF-NOT-FOUND])
# Check for GLUT and set flags accordingly
AC_DEFUN(AM_PATH_GLUT, [
  AC_REQUIRE([AM_LINKRPATH])
  AC_REQUIRE([AM_PATH_GL])
  AC_PROVIDE([AM_PATH_GLUT])

  glut_libs="-lglut"
  glut_cflags="-I/usr/X11/include"
  glut_ldflags="-Wl,${ld_libpath},/usr/X11/lib,${ld_runpath},/usr/X11/lib"

  if test -r ${HOME}/.sblibs ; then
    eval `grep glut_libs= ${HOME}/.sblibs`
    eval `grep glut_cflags= ${HOME}/.sblibs`
    eval `grep glut_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(glut-prefix, [  --with-glut-prefix=PFX  Prefix where GLUT is installed (optional)], [
    glut_cflags="-I$withval/include"
    glut_ldflags="-Wl,${ld_libpath}$withval/lib,${ld_runpath}$withval/lib"
  ], :)

  AC_ARG_WITH(glut-libs, [  --with-glut-libs=ARG    GLUT libraries (optional)], [
    glut_libs="${withval}"
  ], :)

  AC_ARG_WITH(glut-cflags, [  --with-glut-cflags=ARG  GLUT compiler parameters (optional)], [
    glut_cflags="${withval}"
  ], :)

  AC_ARG_WITH(glut-ldflags, [  --with-glut-ldflags=ARG GLUT linker parameters (optional)], [
    glut_ldflags="${withval}"
  ], :)

  saved_LIBS="$LIBS"
  saved_CFLAGS="$CFLAGS"
  saved_LDFLAGS="$LDFLAGS"

  AC_MSG_CHECKING([for GLUT])
  LIBS="$glut_libs $GL_LIBS $saved_LIBS"
  CFLAGS="$glut_cflags $saved_CFLAGS"
  LDFLAGS="$saved_LDFLAGS $glut_ldflags"
  AC_TRY_LINK([ #include <GL/glut.h> ], [ glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); ], [
    AC_MSG_RESULT(yes)
    have_glut=yes

    GLUT_LIBS="$glut_libs"
  ], [
    AC_MSG_RESULT(no)

    AC_MSG_CHECKING([for GLUT with separate libX11 and libXmu])
    LIBS="$glut_libs -lX11 -lXmu $GL_LIBS $saved_LIBS"
    AC_TRY_LINK([ #include <GL/glut.h> ], [ glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); ], [
      AC_MSG_RESULT(yes)
      have_glut=yes

      GLUT_LIBS="$glut_libs -lX11 -lXmu"
    ], [
      AC_MSG_RESULT(no)
      have_glut=no
 
      CFLAGS="$saved_CFLAGS"
      LDFLAGS="$saved_LDFLAGS"
    ])
  ])

  LIBS="$saved_LIBS"

  if test x$have_glut = xno ; then
    GLUT_LIBS=""

    AC_SUBST(GLUT_LIBS)

    ifelse([$2], , :, [$2])
  else
    AC_SUBST(GLUT_LIBS)

    ifelse([$1], , :, [$1])
  fi
])

# AM_SSE2([ACTION-IF-FOUND [,ACTION-IF-NOT-FOUND]])
# Check for Intel(TM) SSE2(TM) capabilities
AC_DEFUN(AM_SSE2,
[
  sse_cflags="-mcpu=pentium4 -march=pentium4 -msse2"
  sse_ldflags="-mcpu=pentium4 -march=pentium4 -msse2"

  if test -r ${HOME}/.sblibs ; then
    eval `grep sse_cflags= ${HOME}/.sblibs`
    eval `grep sse_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(sse2-flags, [--with-sse2-flags=FLAGS     Flags that enable SSE2 compilation (optional)], [
    sse_cflags="$withval"
    sse_ldflags="$withval"
  ], :)

  saved_CFLAGS="$CFLAGS"
  saved_LDFLAGS="$LDFLAGS"

  CFLAGS="$sse_cflags $saved_CFLAGS"
  LDFLAGS="$sse_ldflags $saved_LDFLAGS "
  AC_MSG_CHECKING([for SSE2 support])
  AC_TRY_RUN([
#include <signal.h>
#include <stdlib.h>
typedef float v2df __attribute__((mode(V2DF),aligned(16)));
void panic(int id) { exit(1); }
int main() { volatile v2df x; signal(SIGSEGV, panic); x = x*x; x = __builtin_ia32_sqrtpd(x); return 0; }
  ], [
    AC_MSG_RESULT(yes)
    have_sse=yes
    sse_hack=no
  ], [
    AC_MSG_RESULT(no)

    AC_MSG_CHECKING([for SSE2 support with stack hack])
    AC_TRY_RUN([
#include <signal.h>
#include <stdlib.h>
typedef float v2df __attribute__((mode(V2DF),aligned(16)));
int truemain() __attribute__((noinline));
void panic(int id) { exit(1); }
int truemain() { volatile v2df x; signal(SIGSEGV, panic); x = x*x; x = __builtin_ia32_sqrtpd(x); return 0; }
int main() { asm("movl %esp,%eax"); asm("andl \$ 15,%eax"); asm("subl %esp,%eax"); return truemain(); }
    ], [
      AC_MSG_RESULT(yes)
      have_sse=yes
      sse_hack=yes
    ], [
      AC_MSG_RESULT(no)
      have_sse=no
      sse_hack=no

      CFLAGS="$saved_CFLAGS"
      LDFLAGS="$saved_LDFLAGS"
    ], [
      AC_MSG_RESULT([no, since we are cross-compiling])
    ])
  ], [
    AC_MSG_RESULT([no, since we are cross-compiling])
  ])

  if test x$have_sse = xno ; then
    ifelse([$2], , :, [$2])
  else
    if test x$sse_hack = xyes ; then
      AC_DEFINE(STACK_UNALIGNED, 1, [Defined if stack pointer not aligned to multiples of 16])
    fi
    ifelse([$1], , :, [$1])
  fi
])    

# AM_PATH_PTHREAD([ACTION-IF-FOUND [,ACTION-IF-NOT-FOUND]])
# Check for pthread library and set flags accordingly
AC_DEFUN(AM_PATH_PTHREAD,
[
  pthread_libs="-lpthread"
  pthread_cflags=""
  pthread_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep pthread_libs= ${HOME}/.sblibs`
    eval `grep pthread_cflags= ${HOME}/.sblibs`
    eval `grep pthread_ldflags= ${HOME}/.sblibs`
  fi

  saved_LIBS="$LIBS"
  saved_CFLAGS="$CFLAGS"
  saved_LDFLAGS="$LDFLAGS"

  LIBS="$pthread_libs $saved_LIBS"
  CFLAGS="$pthread_cflags $saved_CFLAGS"
  LDFLAGS="$pthread_ldflags $saved_LDFLAGS "
  AC_MSG_CHECKING([for pthread library])
  AC_TRY_LINK([ #include "pthread.h" ], [ pthread_create(NULL, NULL, NULL, NULL); ], [
    AC_MSG_RESULT(yes)
    have_pthread=yes

    PTHREAD_LIBS="$pthread_libs"

    CFLAGS="$pthread_cflags -mt $saved_CFLAGS"
    AC_MSG_CHECKING([whether -mt works])
    AC_TRY_LINK([ #include "pthread.h" ], [ pthread_create(NULL, NULL, NULL, NULL); ], [
      AC_MSG_RESULT(yes)
      PTHREAD_CFLAGS="-mt"
      CFLAGS="$pthread_cflags $saved_CFLAGS"
    ], [
      AC_MSG_RESULT(no)
      PTHREAD_CFLAGS=""
      CFLAGS="$pthread_cflags $saved_CFLAGS"
    ])
  ], [
    AC_MSG_RESULT(no)
    have_pthread=no

    CFLAGS="$saved_CFLAGS"
    LDFLAGS="$saved_LDFLAGS"
  ])

  LIBS="$saved_LIBS"

  if test x$have_pthread = xno ; then
    PTHREAD_LIBS=""
    PTHREAD_CFLAGS=""

    AC_SUBST(PTHREAD_LIBS)
    AC_SUBST(PTHREAD_CFLAGS)

    ifelse([$2], , :, [$2])
  else
    AC_SUBST(PTHREAD_LIBS)
    AC_SUBST(PTHREAD_CFLAGS)

    ifelse([$1], , :, [$1])
  fi
])    

# AM_PATH_NETCDF([ACTION-IF-FOUND [,ACTION-IF-NOT-FOUND]])
# Check for netcdf library and set flags accordingly
AC_DEFUN(AM_PATH_NETCDF,
[
  AC_REQUIRE([AM_LINKRPATH])
  AC_PROVIDE([AM_PATH_NETCDF])

  netcdf_libs="-lnetcdf"
  netcdf_cflags=""
  netcdf_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep netcdf_libs= ${HOME}/.sblibs`
    eval `grep netcdf_cflags= ${HOME}/.sblibs`
    eval `grep netcdf_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(netcdf-prefix, [--with-netcdf-prefix=PFX    Prefix where libnetcdf is installed (optional)], [
    hierquad_cflags="-I$withval/include"
    hierquad_ldflags="-Wl,${ld_libpath},$withval/lib,${ld_runpath},$withval/lib"
  ], :)

  saved_LIBS="$LIBS"
  saved_CFLAGS="$CFLAGS"
  saved_LDFLAGS="$LDFLAGS"

  LIBS="$netcdf_libs $saved_LIBS"
  CFLAGS="$netcdf_cflags $saved_CFLAGS"
  LDFLAGS="$netcdf_ldflags $saved_LDFLAGS "
  AC_MSG_CHECKING([for NetCDF])
  AC_TRY_LINK([ #include <netcdf.h> ] ,[ nc_create(NULL, 0, NULL) ], [
    AC_MSG_RESULT(yes)
    have_netcdf=yes

    NETCDF_LIBS="$netcdf_libs"
    NETCDF_CFLAGS="$netcdf_cflags"
    NETCDF_LDFLAGS="$netcdf_ldflags"
  ], [
    AC_MSG_RESULT(no)
    have_netcdf=no

    CFLAGS="$saved_CFLAGS"
    LDFLAGS="$saved_LDFLAGS"
  ])

  LIBS="$saved_LIBS"

  if test x$have_netcdf = xno ; then
    NETCDF_LIBS=""
    NETCDF_CFLAGS=""
    NETCDF_LDFLAGS=""

    AC_SUBST(NETCDF_LIBS)
    AC_SUBST(NETCDF_CFLAGS)
    AC_SUBST(NETCDF_LDFLAGS)

    ifelse([$2], , :, [$2])
  else
    AC_SUBST(NETCDF_LIBS)
    AC_SUBST(NETCDF_CFLAGS)
    AC_SUBST(NETCDF_LDFLAGS)

    AC_DEFINE(HAVE_NETCDF, 1, [Defined if NetCDF was found])

    ifelse([$1], , :, [$1])
  fi
])    

# AM_TIMING
# Check for pthread library and set flags accordingly
AC_DEFUN(AM_TIMING,
[
  AC_PROVIDE([AM_TIMING])

  AC_MSG_CHECKING([for _lwp_info])
  AC_TRY_LINK([ #include <sys/lwp.h> ], [ struct lwpinfo x; _lwp_info(&x); ], [
    AC_MSG_RESULT(yes)
    have_lwp=yes
  ], [
    AC_MSG_RESULT(no)
  ])

  AC_MSG_CHECKING([for times])
  AC_TRY_LINK([ #include <sys/times.h> ], [ struct tms x; times(&x); ], [
    AC_MSG_RESULT(yes)
    have_times=yes
  ], [
    AC_MSG_RESULT(no)
  ])

  AC_MSG_CHECKING([for gettimeofday])
  AC_TRY_LINK([ #include <sys/time.h> ], [ struct timeval tp; gettimeofday(&tp, (void *) 0); ], [
    AC_MSG_RESULT(yes)
    have_gettimeofday=yes
  ], [
    AC_MSG_RESULT(no)
  ])

  if test x$have_lwp = xyes ; then
    AC_DEFINE(HAVE_LWP, 1, [Defined if _lwp_info was found])
  fi

  if test x$have_times = xyes ; then
    AC_DEFINE(HAVE_TIMES, 1, [Defined if times was found])
  fi

  if test x$have_gettimeofday = xyes ; then
    AC_DEFINE(HAVE_GETTIMEOFDAY, 1, [Defined if gettimeofday was found])
  fi
])    

# Configuration scripts for GtkGLArea by Janne Lf 1999-17-2
# Some modifications by Steffen Brm

# AM_PATH_GL([ACTION-IF-FOUND [,ACTION-IF-NOT-FOUND]])
# Check for OpenGL or MesaGL and set flags accordingly
AC_DEFUN(AM_PATH_GL,
[
  AC_REQUIRE([AM_LINKRPATH])
  AC_PROVIDE([AM_PATH_GL])

  gl_libs="-lGLU -lGL"
  gl_cflags=""
  gl_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep gl_libs= ${HOME}/.sblibs`
    eval `grep gl_cflags= ${HOME}/.sblibs`
    eval `grep gl_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(gl-prefix, [  --with-gl-prefix=PFX    Prefix where OpenGL or MesaGL is installed (optional)], [
    gl_cflags="-I$withval/include"
    gl_ldflags="-Wl,${ld_libpath}$withval/lib,${ld_runpath}$withval/lib"
  ], :)

  AC_ARG_WITH(gl-libs, [  --with-gl-libs=ARG      OpenGL libraries (optional)], [
    gl_libs="$withval"
  ], :)

  AC_ARG_WITH(gl-cflags, [  --with-gl-cflags=ARG    OpenGL compiler parameters (optional)], [
    gl_cflags="$withval"
  ], :)

  AC_ARG_WITH(gl-ldflags, [  --with-gl-ldflags=ARG   OpenGL linker parameters (optional)], [
    gl_ldflags="$withval"
  ], :)

  saved_LIBS="$LIBS"
  saved_CFLAGS="$CFLAGS"
  saved_LDFLAGS="$LDFLAGS"

  # test for plain OpenGL
  AC_MSG_CHECKING([for GL])
  LIBS="$gl_libs $saved_LIBS"
  CFLAGS="$gl_cflags $saved_CFLAGS"
  LDFLAGS="$gl_ldflags $saved_LDFLAGS"
  AC_TRY_LINK([ #include <GL/gl.h> ], [ glBegin(GL_LINES); glEnd(); ], [
    AC_MSG_RESULT(yes)
    have_GL=yes

    GL_LIBS="$gl_libs"
  ], [
    AC_MSG_RESULT(no)

    AC_MSG_CHECKING([for GL with separate libm])
    LIBS="$gl_libs -lm $saved_LIBS"
  
    AC_TRY_LINK([ #include <GL/gl.h> ], [ glBegin(GL_LINES); glEnd(); ], [
      AC_MSG_RESULT(yes)
      have_GL=yes

      GL_LIBS="$gl_libs -lm"
    ], [
      AC_MSG_RESULT(no)

      AC_MSG_CHECKING([for GL in /usr/X11R6])
      LDFLAGS="$gl_ldflags -Wl,${ld_libpath}/usr/X11R6/lib,${ld_runpath}/usr/X11R6/lib $saved_LDFLAGS"
      CFLAGS="$gl_cflags -I/usr/X11R6/include $saved_CFLAGS"

      AC_TRY_LINK([ #include <GL/gl.h> ], [ glBegin(GL_LINES); glEnd(); ], [
	AC_MSG_RESULT(yes)
        have_GL=yes

        GL_LIBS="$gl_libs"
      ], [
        AC_MSG_RESULT(no)

        # test for Mesa
        AC_MSG_CHECKING([for Mesa])
        LIBS=" -lMesaGLU -lMesaGL -lm $saved_LIBS"
        AC_TRY_LINK([ #include <GL/gl.h> ], [ glBegin(GL_LINES); glEnd(); ], [
          AC_MSG_RESULT(yes)
          have_GL=mesa

          GL_LIBS="-lMesaGLU -lMesaGL -lm"
        ], [
          AC_MSG_RESULT(no)

          # test for Mesa with threads
          AC_MSG_CHECKING([for Mesa with pthreads])
          LIBS="-lMesaGLU -lMesaGL -lm -lpthread $saved_LIBS"
          AC_TRY_LINK([ #include <GL/gl.h> ], [ glBegin(GL_LINES); glEnd(); ], [
            AC_MSG_RESULT(yes)
            have_GL=mesa

	    GL_LIBS="-lMesaGLU -lMesaGL -lm -lpthread"
          ], [
            AC_MSG_RESULT(no)
            have_GL=no
          ])
        ])
      ])
    ])
  ])

  LIBS="$saved_LIBS"
  CFLAGS="$saved_CFLAGS"
  LDFLAGS="$saved_LDFLAGS"

  if test x$have_GL = xno ; then
    GL_LIBS=""

    AC_SUBST(GL_LIBS)

    ifelse([$2], , :, [$2])
  else
    AC_DEFINE(HAVE_GL, 1, [Defined if OpenGL library was found])
    if test x$have_GL = xmesa ; then
      AC_DEFINE(GL_IS_MESAGL, 1, [Defined if GL library is MesaGL])
    fi

    AC_SUBST(GL_LIBS)

    ifelse([$1], , :, [$1])
  fi
])    

# AM_PATH_GTKGL([ACTION-IF-FOUND] [, ACTION-IF-NOT-FOUND])
# Check for GtkGL and set flags accordingly
AC_DEFUN(AM_PATH_GTKGL, [
  AC_REQUIRE([AM_LINKRPATH])
  AC_REQUIRE([AM_PATH_GTK])
  AC_REQUIRE([AM_PATH_GL])
  AC_PROVIDE([AM_PATH_GTKGL])

  gtkgl_libs="-lgtkgl"
  gtkgl_cflags=""
  gtkgl_ldflags=""

  if test -r ${HOME}/.sblibs ; then
    eval `grep gtkgl_libs= ${HOME}/.sblibs`
    eval `grep gtkgl_cflags= ${HOME}/.sblibs`
    eval `grep gtkgl_ldflags= ${HOME}/.sblibs`
  fi

  AC_ARG_WITH(gtkgl-prefix, [  --with-gtkgl-prefix=PFX Prefix where GtkGLArea is installed (optional)], [
    gtkgl_cflags="-I$withval/include"
    gtkgl_ldflags="-Wl,${ld_libpath}$withval/lib,${ld_runpath}$withval/lib"
  ], :)

  AC_ARG_WITH(gtkgl-libs,   [  --with-gtkgl-libs=ARG   GtkGLArea libraries (optional)], [
    gtkgl_libs="$withval"
  ], :)

  AC_ARG_WITH(gtkgl-cflags, [  --with-gtkgl-cflags=ARG GtkGLArea compiler parameters (optional)], [
    gtkgl_cflags="$withval"
  ], :)

  AC_ARG_WITH(gtkgl-ldflags, [  --with-gtkgl-ldflags=ARG GtkGLArea linker parameters (optional)], [
    gtkgl_ldflags="$withval"
  ], :)

  saved_LIBS="$LIBS"
  saved_CFLAGS="$CFLAGS"
  saved_LDFLAGS="$LDFLAGS"

  AC_MSG_CHECKING([for GtkGLArea])
  LIBS="$gtkgl_libs $GTK_LIBS $GL_LIBS $saved_LIBS"
  CFLAGS="$gtkgl_cflags $GTK_CFLAGS $saved_CFLAGS"
  LDFLAGS="$gtkgl_ldflags $saved_LDFLAGS"
  AC_TRY_LINK([ #include <gtkgl/gtkglarea.h> ] ,[ gtk_gl_area_new( (void *) 0 ); ], [
    AC_MSG_RESULT(yes)
    have_gtkgl=yes

    GTKGL_LIBS="$gtkgl_libs"
  ], [
    AC_MSG_RESULT(no)
    have_gtkgl=no

    GTKGL_LIBS=""

    CFLAGS="$saved_CFLAGS"
    LDFLAGS="$saved_LDFLAGS"
  ])

  LIBS="$saved_LIBS"

  if test x$have_gtkgl = xno ; then
    GTKGL_LIBS=""

    AC_SUBST(GTKGL_LIBS)

    ifelse([$2], , :, [$2])
  else
    AC_SUBST(GTKGL_LIBS)

    ifelse([$1], , :, [$1])
  fi
])
